<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.modellist');

class TableBookingModelUsers extends JModelList
{
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'a.id',
				'a.title',
				'a.published'
			);
		}
		parent::__construct($config);
	}
	
	
	protected function populateState($ordering = null, $direction = null) { 
		
		// Initialise variables.
		$app = JFactory::getApplication('administrator');
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_tablebooking');
		$this->setState('params', $params); 

		// List state information.
		parent::populateState('id', 'desc');
	}
	
	protected function getStoreId($id = '') {
		
		// Compile the store id.

		return parent::getStoreId($id);
	}
	
	

	protected function getListQuery() {
		
		$query	= $this->_db->getQuery(true);

		$query->select(
			$this->getState(
				'list.select','a.id, b.name, c.title AS restaurant, a.position '
			)
		);
		$query->from( $this->_db->quoteName('#__tbb_staff') . ' AS a' );
		$query->join('INNER', $this->_db->quoteName('#__users') . ' AS b ON a.id = b.id' );
		$query->join('LEFT', $this->_db->quoteName('#__tbb_restaurants') . ' AS c ON c.id = a.restaurant' );
		$orderCol = $this->getState('list.ordering', 'id');
		$query->order( $this->_db->escape($orderCol).' '.$this->_db->escape($this->getState('list.direction', 'DESC')));

		return $query;
	}


	public function getJusers() {

		$query	= $this->_db->getQuery(true);
		$query->select('a.id as value, a.name as name');
		$query->from($this->_db->quoteName('#__users') . ' AS a');
		$query->where('a.block = 0');
		$query->where($this->_db->quoteName('a.id') . ' NOT IN (SELECT id FROM ' . $this->_db->quoteName('#__tbb_staff') . ')');
		
		$this->_db->setQuery( $query );
		return $this->_db->loadObjectList();
	}


	public function getRestaurants() {

		$query	= $this->_db->getQuery(true);
		$query->select('id, title');
		$query->from('#__tbb_restaurants');

		$this->_db->setQuery( $query );
		return $this->_db->loadObjectList();
	}


	public function addNewUser( $new ) {
		$query	= $this->_db->getQuery(true);

		$columns = array( 
							$this->_db->quoteName('id'),
							$this->_db->quoteName('restaurant'),
							$this->_db->quoteName('position'),
							$this->_db->quoteName('checked_out'),
						);
		$values = (int) $new['user'] . ',' . (int) $new['restaurant'] . ',' . (int) $new['position'] . ', 0';
		$query->insert( $this->_db->quoteName('#__tbb_staff') );
		$query->columns( $columns );
		$query->values( $values );

		$this->_db->setQuery( $query );
		$this->_db->query();
	}


	public function delete( $users ) {

		$query	= $this->_db->getQuery(true);

		$query->delete();
		$query->from( $this->_db->quoteName('#__tbb_staff') );
		$query->where( $this->_db->quoteName('id') . ' IN (' . implode(',',$users) . ')' );

		$this->_db->setQuery( $query );
		$this->_db->query();
	}

}
?>
