<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.controllerform');

class TableBookingControllerBooking extends JControllerForm {

	private $isAjaxRequest = false;
	private $errors = array();


	/**
	 * return in a json format possible table(s) or table combination(s) + possible error 
	 * @return [string] [json format]
	 */
	public function getTables() 
	{
		JSession::checkToken('request') or die('Invalid Token');
	
		$params = array();
		$possibles = array();
		$response = new stdclass;
		$response->error = null;
		$response->html = null;
		$response->selected = '';
		$response->alternatives = array();

		$params['date'] = JRequest::getVar('date', 0, '', 'date');
		$params['start'] = JRequest::getVar('start', 0, '', 'int');
		$params['end'] = JRequest::getVar('end', 0, '', 'int');
		$params['restaurant'] = JRequest::getVar('restaurant', 0, '', 'int');
		$params['places'] = JRequest::getVar('places', 0, '', 'int');
		$params['booking'] = JRequest::getVar('booking', 0, '', 'int');
		$params['view'] = JRequest::getVar('view', 0, '', 'int');
		$restaurant = TbRestaurant::getInstance($params['restaurant']);
		$day = new TbDay( $params['date'] );
		$search = new TbSearch($restaurant, $day, $params['start'], $params['end'], $params['places']);

		// if someone tries to get tables for a different restaurant
		if ( $restaurant->id != $params['restaurant'] )
		{
			exit();
		}

		// in case we are dealing with search for an existing booking
		if ( $params['booking'] )
		{
			$search->setBooking($params['booking']);
		}

		if (!$search->validateInput()) 
		{
			$response->error = $search->error;
		}
		else 
		{
			$possibles = $search->getAvailableTables( $restaurant->getParams()->combinetables );
			
			if (!empty($possibles)) 
			{
				if (is_array($possibles[0])) 
				{
					foreach ($possibles[0] as $final)
					{
                        $response->selected = $response->selected ?
                                $response->selected . ',' . $final->id :
                                $final->id;
					}

				}
				else
				{
					$response->selected = $possibles[0]->id;
				}

				// if display results to client 
				if ($params['view'])
				{
                    $response->html = TbDisplay::showTablesArrangement(
                        $restaurant->getTables(), $possibles, $response->selected
                    );
				}
			}
			else 
			{
				$response->error = JText::_('COM_TABLEBOOKING_ERROR_NO_PLACES_MEET_REQUIREMENTS',true);
			}
		}


		// try to give alternatives
		if (empty($response->selected))
		{
			$length = $params['end'] - $params['start'];
			$start 	= $params['start'] - 3 * $length;
			$end  	= $params['end'] + 3 * $length;
			$workingh = $restaurant->getSchedule($params['date'])->getHours();

			for ($i = $start; $i <= $end; $i = $i + $restaurant->getParams()->timeframe)
			{
				// if hour is not a working hour
				if (!in_array($i, $workingh))
				{
					continue;
				}

				$search = new TbSearch($restaurant, $day, $i, $i+$length, $params['places']);
				if ( $params['booking'] )
				{
					$search->setBooking($params['booking']);
				}

				if (!$search->validateInput()) 
				{
					continue;
				}

				$possibles = $search->getAvailableTables( $restaurant->getParams()->combinetables );

				if (!empty($possibles)) 
				{
					$p = new stdclass;
					$p->idx = $i;
					$p->val = TbDisplay::showTime($i, $restaurant->getParams()->hourformat);
					$response->alternatives[] = $p;
				}
			}
		}	

		echo json_encode($response);
		exit();
	}



	
	

	public function save( $key = 'NULL', $urlVar = NULL ) 
	{
		
		$model = $this->getModel('Booking');
		$form = $model->getForm();
		$data = JRequest::getVar('jform', array(), 'post', 'array');
		$app  = JFactory::getApplication();
		$restaurant = TbRestaurant::getInstance($data['restaurant']);

		// set end hour if client cannot choose end hour
		if ($restaurant->getParams()->booking_length) 
		{
			$data['bookend'] = $data['bookstart'] + $restaurant->getParams()->booking_length;
		}

		// set data in session
		$app->setUserState('com_tablebooking.edit.booking.data', $data);

		// Validate the posted data.
		$return = $model->validate($form, $data);
		
		// Check for validation errors.
		if ($return == false)
		{
			$this->errors = $model->getErrors();
			if(isset($this->errors[0]) && $this->errors[0] instanceof Exception) 
			{
				$message = $this->errors[0]->getMessage();		
			}
			else 
			{
				$message = implode("<br/>", $this->errors);
			}
			
			if (!$this->isAjaxRequest)
			{
				$this->setRedirect( JRoute::_('index.php?option=com_tablebooking&view=booking&layout=edit', false), $message, 'warning' );
			}
			return false;
		}

		// clean data before saving it
		$data = $model->cleanData($data);
		
		// Check if search criterias are correct and if all selected tables are free
		$day 	= new TbDay($data['bookdate']);
		$search = new TbSearch( $restaurant, $day, $data['bookstart'], $data['bookend'], $data['places'] );
		$tables = explode(',', $data['table']);
		if (!empty($data['id'])) 
		{
			$search->setBooking((int) $data['id']);
		}

		if (!$search->validateInput())
		{
			$this->errors = $search->error;
			return false;
		}

		// if remain a table that was chosen but it was not found in the list of available tables
		$available_tables = $search->getAvailableTables(true);
		$chosen = $tables;

		foreach ($available_tables as $available)
		{	
			if(is_array($available)) 
			{
				foreach ($available as $possible) 
				{
					if (in_array($possible->id, $chosen))
					{
						$keys = array_keys($chosen, $possible->id);
						unset($chosen[$keys[0]]);
					}
				}
			}
			else
			{
				if (in_array($available->id, $chosen))
				{
					$keys = array_keys($chosen,$available->id);
					unset($chosen[$keys[0]]);
				}
			}
		}

		if ($chosen)
		{
			$this->errors = JText::_('COM_TABLEBOOKING_SEARCH_ERROR_SELECTED_TABLE_NOT_AVAILABLE');
			return false;
		}
		

		// encode selected table(s) to be saved
		// add restaurant id 
		// set default status for new booking, leave selected status for existing booking
		$data['table'] = json_encode($tables);
		$data['restaurant'] = $restaurant->id;
		$data['status'] = empty($data['id']) ? $restaurant->getParams()->booking_status : $data['status'];
		
		if ($model->save($data)) 
		{	
			// add new booking id into session
			$data['id'] = $model->getState('booking.id');
			$app->setUserState('com_tablebooking.edit.booking.data', $data);
			
			$this->sendNotifications($data,$restaurant);
			
			if (!$this->isAjaxRequest) 
			{
				$this->setRedirect( JRoute::_('index.php?option=com_tablebooking&view=bookings', false), JText::_('COM_TABLEBOOKING_BOOKING_SAVED'), 'message' );
			}
			return true;
		}
	}


	/**
	 * save a booking using an ajax request
	 * data is sent using POST
	 */
	public function saveAjax() 
	{
		JSession::checkToken('request') or die('Invalid Token');

		$this->isAjaxRequest = true;
		$response = new stdclass;
		
		if ($this->save())
		{
			$response->success = 1;
			JFactory::getApplication()->setUserState('com_tablebooking.edit.booking.data', array());
		}
		else
		{
			$response->success = 0;
		}
		$response->errors = $this->errors;

		ob_clean();flush();
		echo json_encode($response);
		exit(0);
	}


	/**
	 * get all hours corresponding to new date when changing the date in the booking form
	 * @return [string] 
	 */
	public function getHours()
	{	
		JSession::checkToken('request') or die('Invalid Token');

		$data = JRequest::getVar('date', '', 'post', 'string');
		if ($data)
		{
			$day = new TbDay($data);
			$restaurant = TbUser::getInstance()->work->restaurant;
			
			// first let's see if a schedule is set for the date
			$schedule = $restaurant->getSchedule($day->date); 
		
			// if a schedule for that certain date doesn't exist default to the schedule for the week day
			if (!$schedule->isSet)
			{
				$schedule = $restaurant->getSchedule($day->date->__get('dayofweek'));
			}
			
			$hours = $schedule->working ? $schedule->getHours(true) : array(''=>'-');
			echo json_encode($hours);
		} 
		else
		{
			echo '0';
		}
		exit();
	}


	/**
	 * return a booking form as the response to the ajax request made in dashboard
	 * set values corresponding to date, hour and table
	 */
	public function getFormAjax()
	{
		$restaurant = TbUser::getInstance()->work->restaurant;
		$date = JRequest::getVar('date','','','string');
		$hour = JRequest::getVar('hour',0,'','int');
		$table = JRequest::getVar('table',0,'','int');

		if (preg_match('/^[0-9]{4}(-[0-9]{2}){2}$/', $date) && (int)$hour != 0 && $table != 0)
		{
			$model = $this->getModel('Booking');
			$form = $model->getForm();
			$form->setValue('restaurant',null,$restaurant->id);
			$form->setValue('bookdate',null,$date);
			$form->setValue('bookstart',null,(int) $hour);
			$form->setValue('table',null,(int) $table);
			$form->setValue('calendar',null, TbDisplay::showDate($date,$restaurant->getParams()->dateformat));
			echo TbDisplay::showBookingForm($form);
		}

		exit(0);
	}


	/*
	 * send email notifications at the end of booking process
	 * send only when location is "client", does not work in the management area
	 * 
	 * @params 	[array]	$data			reservation form values
	 * @params	[object] $restaurant	selected restaurant object
	 */
	protected function sendNotifications($data,$restaurant)
	{
		$location = JRequest::getVar('location','','','string');
		$validator = new emailValidator($data,$restaurant->getParams());
		
		if ($location == 'client')
		{
			if ($validator->validateForReceive())
			{
				$validator->getMailer()->sendNotification();
			}
			else
			{
				$errors = $validator->getErrors();
				foreach ($errors as $error)
				{
					$this->errors[] = $error;
				}
			}
			
			if ($validator->validateForSending())
			{
				$validator->getMailer()->sendNotification();
			}
			else
			{
				$errors = $validator->getErrors();
				foreach ($errors as $error)
				{
					$this->errors[] = $error;
				}
			}
		}
	}
	
	
	/*
	 * booking confirmation by client
	 */
	public function confirmBooking() 
	{
		$key = JRequest::getVar('key','','','string');
		if ($key)
		{
			 $parts = explode(':', $key);
			 if (isset($parts[1]))
			 {
				 $id = $parts[0];
				 $this->getModel()->changeBookingStatus($id,$parts[1],1);
			 }
		}
		
		$this->setRedirect(JRoute::_('index.php?option=com_tablebooking&view=search',false), JText::_('COM_TABLEBOOKING_CONFIRMATION_FINISHED'), 'message');
	}
	
	
	/*
	 * booking cancelation by client
	 */
	public function cancelBooking() 
	{
		$key = JRequest::getVar('key','','','string');
		if ($key)
		{
			 $parts = explode(':', $key);
			 if (isset($parts[1]))
			 {
				 $id = $parts[0];
				 $this->getModel()->changeBookingStatus($id,$parts[1],2);
			 }
		}
		
		$this->setRedirect(JRoute::_('index.php?option=com_tablebooking&view=search',false), JText::_('COM_TABLEBOOKING_CANCELATION_FINISHED'), 'message');
	}


	public function close()
	{
		$this->setRedirect(JRoute::_('index.php?option=com_tablebooking&view=bookings',false));
	}
}

?>
