<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class emailValidator
{
	
	protected $errors = array();
	protected $booking_details;
	protected $restaurant_params;
	protected $mailer;
	
	public function __construct($booking_details,$restaurant_params)
	{
		$this->booking_details = $booking_details;
		$this->restaurant_params = $restaurant_params;
		$this->mailer = new stdclass;
	}

	// RECEIVE NOTIFICATIONS
	protected function validateReceiveEmailAddress()
	{
		if (empty($this->restaurant_params->receive_email_to) || !$this->validateEmailAddress($this->restaurant_params->receive_email_to))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_RECEIVE_EMAIL');
		}
	}
	
	protected function validateReceiveSubject()
	{
		if (empty($this->restaurant_params->receive_email_subject))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_RECEIVE_NOTIFICATION_SUBJECT');
		}
	}
	
	
	protected function validateReceiveBody()
	{
		if (empty($this->restaurant_params->receive_email_body))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_RECEIVE_NOTIFICATION_BODY');
		}
	}
	// /RECEIVE NOTIFICATIONS
	
	
	// SEND NOTIFICATION
	protected function validateSendFromName()
	{
		if (empty($this->restaurant_params->send_email_fromname))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_SENDFROM_NAME');
		}
	}
	
	protected function validateSendSubject()
	{
		if (empty($this->restaurant_params->send_email_subject))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_SEND_NOTIFICATION_SUBJECT');
		}
	}
	
	
	protected function validateSendBody()
	{
		if (empty($this->restaurant_params->send_email_body))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_SEND_NOTIFICATION_BODY');
		}
	}
	
	
	protected function validateCustomerEmailAddress()
	{
		if (empty($this->booking_details['email']) || !$this->validateEmailAddress($this->booking_details['email']))
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_INVALID_CUSTOMER_EMAIL');
		}
	}
	// /SEND NOTIFICATION
	
	
	
	protected function validateEmailAddress($email)
	{
		if (empty($email) || !filter_var($email,FILTER_VALIDATE_EMAIL))
		{
			return false;
		}
		return true;
	}
	
	
	/*
	 * check if option for receiving a notification about new reservation is enabled
	 * also check if all data necessary to send the notification is ok
	 * if ok instantiate the mailer
	 * 
	 * @return boolean 	true if everythging ok, false if disabled/missing data
	 */
	public function validateForReceive() 
	{	
		$this->errors = array();
		if ($this->restaurant_params->receive_email) 
		{
			$this->validateReceiveEmailAddress();
			$this->validateReceiveSubject();
			$this->validateReceiveBody();
		}
		else
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_RECEIVE_NOTIFICATION_DISABLED', true);
		}
		
		if (empty($this->errors))
		{
			$mailfrom = JFactory::getApplication()->getCfg('mailfrom');
			$fromname = JFactory::getApplication()->getCfg('fromname');
			$subject = TbDisplay::stripNotificationsTags($this->restaurant_params->receive_email_subject, $this->booking_details, $this->restaurant_params);
			$body = TbDisplay::stripNotificationsTags($this->restaurant_params->receive_email_body, $this->booking_details, $this->restaurant_params);
			$this->mailer = new TbMailer($this->restaurant_params->receive_email_to, $subject, $body, $mailfrom, $fromname, (bool) $this->restaurant_params->email_format);
			return true;
		}
		return false;
	}
	
	
	/*
	 * verify if option to send email to client at the end of booking process is enabled
	 * also check if all we have all the data necessary for sending the email to the client
	 * if ok instantiate the mailer
	 * 
	 * @return boolean 	true if everythging ok, false if disabled/missing data
	 */
	public function validateForSending()
	{
		$this->errors = array();
		if ($this->restaurant_params->send_email) 
		{
			$this->validateSendFromName();
			$this->validateSendSubject();
			$this->validateSendBody();
			$this->validateCustomerEmailAddress();
		}
		else
		{
			$this->errors[] = JText::_('COM_TABLEBOOKING_SENDING_NOTIFICATION_DISABLED', true);
		}

		if (empty($this->errors))
		{
			$mailfrom = JFactory::getApplication()->getCfg('mailfrom');
			$fromname = $this->restaurant_params->send_email_fromname;
			$subject = TbDisplay::stripNotificationsTags($this->restaurant_params->send_email_subject, $this->booking_details, $this->restaurant_params);
			$body = TbDisplay::stripNotificationsTags($this->restaurant_params->send_email_body, $this->booking_details, $this->restaurant_params);
			$this->mailer = new TbMailer($this->booking_details['email'], $subject, $body, $mailfrom, $fromname, (bool) $this->restaurant_params->email_format);
			return true;
		}
		return false;
	}
	
	
	// instantiate the mailer
	public function getMailer()
	{
		return $this->mailer;
	}
	
	
	public function getErrors()
	{
		return $this->errors;
	}
	

}
